import matplotlib.pyplot as plt
import numpy as np


def create_pascal_label_colormap():
    colormap = np.zeros((256, 3), dtype=int)
    ind = np.arange(256, dtype=int)
    for shift in reversed(range(8)):
        for channel in range(3):
            colormap[:, channel] |= ((ind >> channel) & 1) << shift
        ind >>= 3
    return colormap

def label_to_color_image(label):
    if label.ndim != 2:
        raise ValueError('Expect 2-D input label')
    colormap = create_pascal_label_colormap()
    if np.max(label) >= len(colormap):
        raise ValueError('label value too large.')
    return colormap[label]

def vis_segmentation(image, seg_map, name='demo.jpg', if_save=False, if_show=True):

    seg_image = label_to_color_image(seg_map).astype(np.uint8)
    plt.figure()
    plt.imshow(image)
    plt.imshow(seg_image, alpha=0.7)
    plt.axis('off')

    if if_save:
        plt.savefig("vis_{}".format(name))

    if if_show:
        plt.show()


import moyan
import os
import shutil
import random
import cv2
import numpy as np 
from PIL import Image
import matplotlib.pyplot as plt

img_dir = r"D:\code\PAI\docUnet.pytorch\docUnet.pytorch-master\image_generate_crop"
output_dir = r"C:\Users\Moyan\Desktop\tabel_mask"


img_list = moyan.walkDir2RealPathList(img_dir, filter_postfix=[".png"])
# random.shuffle(img_list)
for idx, path in enumerate(img_list):
    # print(idx, path)
    md5_name = moyan.get_file_md5(path)
    print(idx, md5_name)
    new_path = os.path.join(output_dir, md5_name+".png")
    shutil.copyfile(path, new_path)


# img_dir = r"C:\Users\Moyan\Desktop\img"
# output_dir = r"C:\Users\Moyan\Desktop\img"
# img_list = moyan.walkDir2RealPathList(img_dir, filter_postfix=[".bmp"])
# for idx, path in enumerate(img_list):
#     print(idx, path)
#     im = moyan.cv_read(path, 0)
#     im = cv2.resize(im, (600, 800))
#     new_path = os.path.splitext(path)[0] + ".jpg"
#     cv2.imwrite(new_path, im)

    # exit()

# img_dir = r"D:\Dataset\TableBank_data\Detection_data\cropbox\select_useful2\img"
# out_dir = r"D:\Dataset\TableBank_data\Detection_data\cropbox\select_useful2\padding_img"

# img_list = os.listdir(img_dir)

# def resize_img_keep_ratio(img_name,target_size):
#     img = cv2.imread(img_name)
#     old_size= img.shape[0:2]
#     #ratio = min(float(target_size)/(old_size))
#     ratio = min(float(target_size[i])/(old_size[i]) for i in range(len(old_size)))
#     new_size = tuple([int(i*ratio) for i in old_size])
#     img = cv2.resize(img,(new_size[1], new_size[0]))
#     pad_w = target_size[1] - new_size[1]
#     pad_h = target_size[0] - new_size[0]
#     top,bottom = pad_h//2, pad_h-(pad_h//2)
#     left,right = pad_w//2, pad_w -(pad_w//2)
#     img_new = cv2.copyMakeBorder(img,top,bottom,left,right,cv2.BORDER_CONSTANT,None,(255, 255, 255))
#     return img_new

# def resize_img_keep_ratio2(img_name,target_size):
#     img = cv2.imread(img_name)
#     old_size= img.shape[0:2]
#     #ratio = min(float(target_size)/(old_size))
    
#     ratio = min(float(target_size[i])/(old_size[i]) for i in range(len(old_size)))
#     new_size = tuple([int(i*ratio) for i in old_size])
#     img = cv2.resize(img,(new_size[1], new_size[0]))
#     pad_w = target_size[1] - new_size[1]
#     pad_h = target_size[0] - new_size[0]
#     top,bottom = pad_h//2, pad_h-(pad_h//2)
#     left,right = pad_w//2, pad_w -(pad_w//2)
#     img_new = cv2.copyMakeBorder(img,top,bottom,left,right,cv2.BORDER_CONSTANT,None,(255, 255, 255))
#     return img_new

# for idx, names in enumerate(img_list):
#     print(idx, names)
#     img_path = os.path.join(img_dir, names)
#     out_path = os.path.join(out_dir, names)
#     # im = cv2.imread(img_path, 0)
#     new_im = resize_img_keep_ratio(img_path, (800, 600))
#     cv2.imwrite(out_path, new_im)
#     # exit()



# npy_path = r"D:\code\PAI\docUnet.pytorch\docUnet.pytorch\img\padding_mask_aug\%5BMS-OXCFXICS%5D-080425_33#84_72_527_479_0.jpg.npy"
# label = np.load(npy_path)

# # print(label[:, :, 0]

# for i in range(100): 
#     print("第110行第{}个: ({}, {})".format(i, label[10][i][0], label[10][i][1]))


# -*- coding:utf-8 -*-
# time: 2020-08-17
# @author: moyan

import json
import io, math, os
import uuid
# from moyan.tools import pathExit
from tqdm import tqdm
# from labelme import utils
import base64
import numpy as np
import PIL.ImageDraw
import PIL.ExifTags
import PIL.Image
import PIL.ImageOps

global label_name_to_value
label_name_to_value = {'_background_': 0}

def img_data_to_pil(img_data):
    f = io.BytesIO()
    f.write(img_data)
    img_pil = PIL.Image.open(f)
    return img_pil


def img_data_to_arr(img_data):
    img_pil = img_data_to_pil(img_data)
    img_arr = np.array(img_pil)
    return img_arr

def img_b64_to_arr(img_b64):
    img_data = base64.b64decode(img_b64)
    img_arr = img_data_to_arr(img_data)
    return img_arr

def shape_to_mask(
    img_shape, points, shape_type=None, line_width=10, point_size=5
):
    mask = np.zeros(img_shape[:2], dtype=np.uint8)
    mask = PIL.Image.fromarray(mask)
    draw = PIL.ImageDraw.Draw(mask)
    xy = [tuple(point) for point in points]
    if shape_type == "circle":
        assert len(xy) == 2, "Shape of shape_type=circle must have 2 points"
        (cx, cy), (px, py) = xy
        d = math.sqrt((cx - px) ** 2 + (cy - py) ** 2)
        draw.ellipse([cx - d, cy - d, cx + d, cy + d], outline=1, fill=1)
    elif shape_type == "rectangle":
        assert len(xy) == 2, "Shape of shape_type=rectangle must have 2 points"
        draw.rectangle(xy, outline=1, fill=1)
    elif shape_type == "line":
        assert len(xy) == 2, "Shape of shape_type=line must have 2 points"
        draw.line(xy=xy, fill=1, width=line_width)
    elif shape_type == "linestrip":
        draw.line(xy=xy, fill=1, width=line_width)
    elif shape_type == "point":
        assert len(xy) == 1, "Shape of shape_type=point must have 1 points"
        cx, cy = xy[0]
        r = point_size
        draw.ellipse([cx - r, cy - r, cx + r, cy + r], outline=1, fill=1)
    else:
        assert len(xy) > 2, "Polygon must have points more than 2"
        draw.polygon(xy=xy, outline=1, fill=1)
    mask = np.array(mask, dtype=bool)
    return mask


def shapes_to_label(img_shape, shapes, label_name_to_value, line_width):
    cls = np.zeros(img_shape[:2], dtype=np.int32)
    ins = np.zeros_like(cls)
    instances = []
    for shape in shapes:
        points = shape["points"]
        label = shape["label"]
        group_id = shape.get("group_id")
        if group_id is None:
            group_id = uuid.uuid1()
        shape_type = shape.get("shape_type", None)

        cls_name = label
        instance = (cls_name, group_id)

        if instance not in instances:
            instances.append(instance)
        ins_id = instances.index(instance) + 1
        cls_id = label_name_to_value[cls_name]

        mask = shape_to_mask(img_shape[:2], points, shape_type, line_width=line_width)
        cls[mask] = cls_id
        ins[mask] = ins_id

    return cls, ins

def lblsave(filename, lbl):
    import imgviz

    if os.path.splitext(filename)[1] != ".png":
        filename += ".png"
    # Assume label ranses [-1, 254] for int32,
    # and [0, 255] for uint8 as VOC.
    if lbl.min() >= -1 and lbl.max() < 255:
        lbl_pil = PIL.Image.fromarray(lbl.astype(np.uint8), mode="P")
        colormap = imgviz.label_colormap()
        lbl_pil.putpalette(colormap.flatten())
        lbl_pil.save(filename)
    else:
        raise ValueError(
            "[%s] Cannot save the pixel-wise class label as PNG. "
            "Please consider using the .npy format." % filename
        )


def labelme_json_to_png(jsonlist, img_dir, save_dir, pix_line=None):

    if not os.path.exists(save_dir):
        print("create new folder: {}".format(save_dir))
        os.makedirs(save_dir)

    for json_path in tqdm(jsonlist):
        assert os.path.exists(json_path)
        name = os.path.splitext(os.path.basename(json_path))[0]
        png_save_path = os.path.join(save_dir, name.strip()+'.png')

        data = json.load(open(json_path))
        imageData = data.get('imageData')
        if not imageData:
            img_path = os.path.join(img_dir, name.strip()+'.jpg')
            assert os.path.exists(img_path)
            with open(img_path, 'rb') as f:
                imageData = f.read()
                imageData = base64.b64encode(imageData).decode('utf-8')
        img = img_b64_to_arr(imageData)

        if not pix_line:
            h, w = img.shape[:2]
            pix_all = h*w
            if pix_all < 600*600:
                line_width = 1
            elif pix_all < 1200*1200:
                line_width = 2
            else:
                line_width = 3
        else:
            line_width = pix_line

        for shape in sorted(data['shapes'], key=lambda x: x['label']):
            label_name = shape['label']
            if label_name in label_name_to_value:
                label_value = label_name_to_value[label_name]
            else:
                label_value = len(label_name_to_value)
                label_name_to_value[label_name] = label_value

        lbl, _ = shapes_to_label(img.shape, data['shapes'], label_name_to_value, line_width=line_width)
        lblsave(png_save_path, lbl)
               

def main():
    def walkDir2RealPathList(path, filter_postfix=[]):
        root_lists = []
        filter_postfix = filter_postfix
        if filter_postfix:
            print("Files will be searched by the specified suffix, {}".format(filter_postfix))
        else:
            print("All files will be searched")

        for fpathe, dirs, fs in os.walk(path):
            # 返回的是一个三元tupple(dirpath, dirnames, filenames),
            for f in fs:
                # print(os.path.join(fpathe, f))
                apath = os.path.join(fpathe, f)
                ext = os.path.splitext(apath)[1]
                if filter_postfix:
                    if ext in filter_postfix:
                        root_lists.append(apath)
                else:
                    root_lists.append(apath)
        return root_lists
    imgdir = r'D:\code\segcode-pytorch\data\data_example\json'
    jsonlist = walkDir2RealPathList(imgdir, filter_postfix=['.json'])
    labelme_json_to_png(jsonlist, imgdir, save_dir=imgdir)

if __name__ == "__main__":
    main()


# coding:utf-8
"""
python2
Copyright: https://github.com/bgshih/crnn/blob/master/tool/create_dataset.py
"""
import os
import lmdb # install lmdb by "pip install lmdb"
import cv2
import numpy as np
from tools import readTxt2Lines


def checkImageIsValid(imageBin):
    if imageBin is None:
        return False
    imageBuf = np.fromstring(imageBin, dtype=np.uint8)
    img = cv2.imdecode(imageBuf, cv2.IMREAD_GRAYSCALE)
    imgH, imgW = img.shape[0], img.shape[1]
    if imgH * imgW == 0:
        return False
    return True


def writeCache(env, cache):
    with env.begin(write=True) as txn:
        for k, v in cache.iteritems():
            txn.put(k, v)


def create_cls_dataset(outputPath, imglabel_csv, lexiconList=False, checkValid=True):
    """
    Create LMDB dataset for CRNN training.
    ARGS:
        outputPath    : LMDB output path
        imagePathList : list of image path
        labelList     : list of corresponding groundtruth texts
        lexiconList   : (optional) list of lexicon lists
        checkValid    : if true, check the validity of every image
    """

    assert(os.path.exists(imglabel_csv))
    csv_lists = readTxt2Lines(imglabel_csv)
    imagePathList = []
    labelList = []
    for line in csv_lists:
        imgPath, gt = line.strip().split('	')
        imagePathList.append(imgPath.strip())
        labelList.append(gt.strip())
        
    assert(len(imagePathList) == len(labelList))
    nSamples = len(imagePathList)
    env = lmdb.open(outputPath, map_size=1099511627776)
    cache = {}
    cnt = 1
    for i in range(nSamples):
        imagePath = imagePathList[i]
        label = labelList[i]
        if not os.path.exists(imagePath):
            print('%s does not exist' % imagePath)
            continue
        with open(imagePath, 'r') as f:
            imageBin = f.read()
        if checkValid:
            if not checkImageIsValid(imageBin):
                print('%s is not a valid image' % imagePath)
                continue

        imageKey = 'image-%09d' % cnt
        labelKey = 'label-%09d' % cnt
        cache[imageKey] = imageBin
        cache[labelKey] = label
        if lexiconList:
            lexiconKey = 'lexicon-%09d' % cnt
            cache[lexiconKey] = ' '.join(lexiconList[i])
        if cnt % 1000 == 0:
            writeCache(env, cache)
            cache = {}
            print('Written %d / %d' % (cnt, nSamples))

        cnt += 1
    nSamples = cnt-1
    cache['num-samples'] = str(nSamples)
    writeCache(env, cache)
    print('Created dataset with %d samples' % nSamples)


if __name__ == '__main__':
    create_cls_dataset(
        imglabel_csv='/data/code/sunchao/idt-pitaya-serv-pytorch/data/0717_test/abs_hard.csv',
        outputPath='./data/pricetag_0717_hardtest',
    )


# -*- coding: utf-8 -*-
# Created on 8月-26-20 15:28
# @site: https://github.com/moyans
# @author: moyan
from __future__ import absolute_import
from __future__ import print_function
import cv2
import json
import os
import xml.dom.minidom
from .xml_utils import create_child_node, create_object_node, write_xml_file


COCO_Year = '2020'
AUTHOR = 'Moyan'
SEGMENTED = '0'

def coco2voc(input_json, input_img_dir, output_xml_dir):

    with open(input_json, "r+") as f:
        label_dicts = json.load(f)
        annotations = label_dicts["annotations"]
        categories = label_dicts['categories']
        images = label_dicts['images']

    img2id = {}
    for nn in images:
        img2id[nn['file_name']]= str(nn['id'])

    id2class = {}
    for item in categories:
        id2class[item['id']] = item['name']

    processed_labels = []
    for i in annotations:
        if (i['category_id'] in id2class.keys()):
            processed_labels.append({
                "filename": str(i["image_id"]),
                "name": id2class[i["category_id"]],
                "bndbox": i["bbox"]
            })


    img_list = os.listdir(input_img_dir)
    if img_list == 0:
        print("Do not find images in your img_path")
        os._exit(-1)

    if not os.path.exists(output_xml_dir):
        os.mkdir(output_xml_dir)

    for idx, img_name in enumerate(img_list):
        save_name = img_name[:-4]
        print(idx, save_name)
        xml_file_name = os.path.join(output_xml_dir, (save_name + '.xml'))

        try:
            img = cv2.imread(os.path.join(input_img_dir, img_name))
            height, width, channel = img.shape

            my_dom = xml.dom.getDOMImplementation()
            doc = my_dom.createDocument(None, 'annotation', None)
            root_node = doc.documentElement

            create_child_node(doc, 'folder', input_img_dir, root_node)
            create_child_node(doc, 'filename', img_name, root_node)
            source_node = doc.createElement('source')
            create_child_node(doc, 'database', 'LOGODection', source_node)
            create_child_node(doc, 'annotation', 'COCO' + COCO_Year, source_node)

            create_child_node(doc, 'image', 'flickr', source_node)
            create_child_node(doc, 'flickrid', 'NULL', source_node)
            root_node.appendChild(source_node)
            owner_node = doc.createElement('owner')
            create_child_node(doc, 'flickrid', 'NULL', owner_node)
            create_child_node(doc, 'name', AUTHOR, owner_node)
            root_node.appendChild(owner_node)

            size_node = doc.createElement('size')
            create_child_node(doc, 'width', str(width), size_node)
            create_child_node(doc, 'height', str(height), size_node)
            create_child_node(doc, 'depth', str(channel), size_node)
            root_node.appendChild(size_node)
            create_child_node(doc, 'segmented', SEGMENTED, root_node)

            for label in processed_labels:
                if img2id[img_name] == label["filename"]:
                    object_node = create_object_node(doc, label)
                    root_node.appendChild(object_node)
                    write_xml_file(doc, xml_file_name)
        except AttributeError:
            print('error in {}'.format(os.path.join(input_img_dir, img_name)))



def xywh_to_xyxy(xywh):
    """Convert [x1 y1 w h] box format to [x1 y1 x2 y2] format."""
    if isinstance(xywh, (list, tuple)):
        # Single box given as a list of coordinates
        assert len(xywh) == 4
        x1, y1 = xywh[0], xywh[1]
        x2 = x1 + np.maximum(0., xywh[2] - 1.)
        y2 = y1 + np.maximum(0., xywh[3] - 1.)
        return (x1, y1, x2, y2)
    elif isinstance(xywh, np.ndarray):
        # Multiple boxes given as a 2D ndarray
        return np.hstack(
            (xywh[:, 0:2], xywh[:, 0:2] + np.maximum(0, xywh[:, 2:4] - 1))
        )
    else:
        raise TypeError('Argument xywh must be a list, tuple, or numpy array.')

def xyxy_to_xywh(xyxy):
    """Convert [x1 y1 x2 y2] box format to [x1 y1 w h] format."""
    if isinstance(xyxy, (list, tuple)):
        # Single box given as a list of coordinates
        assert len(xyxy) == 4
        x1, y1 = xyxy[0], xyxy[1]
        w = xyxy[2] - x1 + 1
        h = xyxy[3] - y1 + 1
        return (x1, y1, w, h)
    elif isinstance(xyxy, np.ndarray):
        # Multiple boxes given as a 2D ndarray
        return np.hstack((xyxy[:, 0:2], xyxy[:, 2:4] - xyxy[:, 0:2] + 1))
    else:
        raise TypeError('Argument xyxy must be a list, tuple, or numpy array.')


# -*- coding: utf-8 -*-
# Created on 8月-27-20 16:17
# @site: https://github.com/moyans
# @author: moyan
    
import os
import moyan


def main():
    root_dir = r'D:\Dataset\WiderPerson'
    txt_dir = os.path.join(root_dir, 'Annotations')
    img_dir = os.path.join(root_dir, 'Images')
    xml_dir = os.path.join(root_dir, 'xml')
    txt_list = os.listdir(txt_dir)
    for idx, names in enumerate(txt_list):
        img_name = os.path.splitext(names)[0]
        txt_path = os.path.join(txt_dir, names)
        img_path = os.path.join(img_dir, img_name)
        assert os.path.exists(txt_path)
        assert os.path.exists(img_path)
        print(idx, img_name)
        objects = []
        pic_struct = {}
        height, width, channel = moyan.cv_read(img_path).shape
        pic_struct['width'] = str(width)
        pic_struct['height'] = str(height)
        pic_struct['depth'] = str(channel)
        objects.append(pic_struct)

        txt_lines = moyan.readTxt2Lines(txt_path)
        for k, v in enumerate(txt_lines):
            if k == 0 : continue
            class_label, x1, y1, x2, y2 =  v.strip().split(' ')
            if int(class_label) == 5 : continue
            obj_struct = {}
            # print(class_label, x1, y1, x2, y2)
            obj_struct['name'] = 'person'
            obj_struct['bbox'] = [int(float(x1)),
                              int(float(y1)),
                              int(float(x2)),
                              int(float(y2))]
            objects.append(obj_struct)
        xml_path = os.path.join(xml_dir, os.path.splitext(img_name.strip())[0]+'.xml')
        moyan.writeXml(objects, img_name, xml_path)
        # print(objects, xml_path)
    
if __name__ == '__main__':
    main()


DIFFICULT = '0'
TRUNCATED = '0'
POSE = 'Unspecified'


def create_element_node(doc, tag, attr):
    element_node = doc.createElement(tag)
    text_node = doc.createTextNode(attr)
    element_node.appendChild(text_node)
    return element_node


def create_child_node(doc, tag, attr, parent_node):
    child_node = create_element_node(doc, tag, attr)
    parent_node.appendChild(child_node)


def create_object_node(doc, attrs):
    object_node = doc.createElement('object')
    create_child_node(doc, 'name', attrs['name'], object_node)
    create_child_node(doc, 'pose', POSE, object_node)
    create_child_node(doc, 'truncated', TRUNCATED, object_node)
    create_child_node(doc, 'difficult', DIFFICULT, object_node)
    bndbox_node = doc.createElement('bndbox')
    create_child_node(doc, 'xmin', str(int(attrs['bndbox'][0])), bndbox_node)
    create_child_node(doc, 'ymin', str(int(attrs['bndbox'][1])), bndbox_node)
    create_child_node(doc, 'xmax', str(int(attrs['bndbox'][0] + attrs['bndbox'][2])), bndbox_node)
    create_child_node(doc, 'ymax', str(int(attrs['bndbox'][1] + attrs['bndbox'][3])), bndbox_node)
    object_node.appendChild(bndbox_node)
    return object_node


def write_xml_file(doc, filename):
    tmpfile = open('tmp.xml', 'w')
    doc.writexml(tmpfile, addindent='' * 4, newl='\n', encoding='utf-8')
    tmpfile.close()
    fin = open('tmp.xml')
    fout = open(filename, 'w')
    lines = fin.readlines()
    for line in lines[1:]:
        if line.split():
            fout.writelines(line)
    fin.close()
    fout.close()


# import torch
# import numpy as np


# class Cutout(object):
#     """Randomly mask out one or more patches from an image.
#     Args:
#         n_holes (int): Number of patches to cut out of each image.
#         length (int): The length (in pixels) of each square patch.
#     """
#     def __init__(self, n_holes=1, length=14):
#         self.n_holes = n_holes
#         self.length = length

#     def __call__(self, img):
#         """
#         Args:
#             img (Tensor): Tensor image of size (C, H, W).
#         Returns:
#             Tensor: Image with n_holes of dimension length x length cut out of it.
#         """
#         h = img.size(1)
#         w = img.size(2)

#         mask = np.ones((h, w), np.float32)

#         for n in range(self.n_holes):
#         	# (x,y)表示方形补丁的中心位置
#             y = np.random.randint(h)
#             x = np.random.randint(w)

#             y1 = np.clip(y - self.length // 2, 0, h)
#             y2 = np.clip(y + self.length // 2, 0, h)
#             x1 = np.clip(x - self.length // 2, 0, w)
#             x2 = np.clip(x + self.length // 2, 0, w)

#             mask[y1: y2, x1: x2] = 0.

#         mask = torch.from_numpy(mask)
#         mask = mask.expand_as(img)
#         img = img * mask

#         return img


# -*- coding: utf-8 -*-
# Created on 8月-26-20 14:45
# @site: https://github.com/moyans
# @author: moyan
import os
import json
import xml.etree.ElementTree as ET

def readTxt2Lines(txt_file_path):
    with open(txt_file_path, 'r') as f:
        lines = f.readlines()
        lines = [line.strip() for line in lines]
        return lines

def writeLines2Txt(lines, name):
    f = open(name, 'w')
    temp = ''
    for idx, name in enumerate(lines):
        if idx == len(lines)-1:
            temp += name
        else:
            temp += name + '\n'
    f.write(temp)
    f.close()

coco = dict()
coco['images'] = []
coco['type'] = 'instances'
coco['annotations'] = []
coco['categories'] = []

category_set = dict()
image_set = set()

category_item_id = 0
image_id = 20200000000
annotation_id = 0

voc_txt_lines = []

# coco['categories'] = [{'supercategory': 'none', 'id': 1, 'name': 'person'}, ]
# category_set = {'person': 1,}


def addCatItem(name):
    global category_item_id
    category_item = dict()
    category_item['supercategory'] = 'none'
    category_item_id += 1
    category_item['id'] = category_item_id
    category_item['name'] = name
    coco['categories'].append(category_item)
    category_set[name] = category_item_id
    return category_item_id


def addImgItem(file_name, size):
    global image_id
    if file_name is None:
        raise Exception('Could not find filename tag in xml file.')
    if size['width'] is None:
        raise Exception('Could not find width tag in xml file.')
    if size['height'] is None:
        raise Exception('Could not find height tag in xml file.')
    image_id += 1
    image_item = dict()
    image_item['id'] = image_id
    image_item['file_name'] = file_name

    im_name = file_name.strip().split('.j')[0]
    voc_txt_lines.append(im_name)

    image_item['width'] = size['width']
    image_item['height'] = size['height']
    coco['images'].append(image_item)
    image_set.add(file_name)
    return image_id


def addAnnoItem(object_name, image_id, category_id, bbox):
    global annotation_id
    annotation_item = dict()
    annotation_item['segmentation'] = []
    seg = []
    # bbox[] is x,y,w,h
    # left_top
    seg.append(bbox[0])
    seg.append(bbox[1])
    # left_bottom
    seg.append(bbox[0])
    seg.append(bbox[1] + bbox[3])
    # right_bottom
    seg.append(bbox[0] + bbox[2])
    seg.append(bbox[1] + bbox[3])
    # right_top
    seg.append(bbox[0] + bbox[2])
    seg.append(bbox[1])

    annotation_item['segmentation'].append(seg)

    annotation_item['area'] = bbox[2] * bbox[3]
    annotation_item['iscrowd'] = 0
    annotation_item['ignore'] = 0
    annotation_item['image_id'] = image_id
    annotation_item['bbox'] = bbox
    annotation_item['category_id'] = category_id
    annotation_id += 1
    annotation_item['id'] = annotation_id
    coco['annotations'].append(annotation_item)


def parseXmlFiles(xml_dir, txt_list):
    for f in txt_list:
        xml_file = os.path.join(xml_dir, f.strip()+'.xml')
        assert os.path.exists(xml_file)
        print(xml_file)

        bndbox = dict()
        size = dict()
        current_image_id = None
        current_category_id = None
        file_name = None
        size['width'] = None
        size['height'] = None
        size['depth'] = None
        
        tree = ET.parse(xml_file)
        root = tree.getroot()
        if root.tag != 'annotation':
            raise Exception('pascal voc xml root element should be annotation, rather than {}'.format(root.tag))

        # elem is <folder>, <filename>, <size>, <object>
        for elem in root:
            current_parent = elem.tag
            current_sub = None
            object_name = None

            if elem.tag == 'folder':
                continue

            if elem.tag == 'filename':
                file_name = elem.text
                if file_name in category_set:
                    raise Exception('file_name duplicated')

            # add img item only after parse <size> tag
            elif current_image_id is None and file_name is not None and size['width'] is not None:
                if file_name not in image_set:
                    current_image_id = addImgItem(file_name, size)
                    print('add image with {} and {}'.format(file_name, size))
                else:
                    raise Exception('duplicated image: {}'.format(file_name))
                    # subelem is <width>, <height>, <depth>, <name>, <bndbox>
            for subelem in elem:
                bndbox['xmin'] = None
                bndbox['xmax'] = None
                bndbox['ymin'] = None
                bndbox['ymax'] = None

                current_sub = subelem.tag
                if current_parent == 'object' and subelem.tag == 'name':
                    object_name = subelem.text
                    if object_name not in category_set:
                        current_category_id = addCatItem(object_name)
                    else:
                        current_category_id = category_set[object_name]

                elif current_parent == 'size':
                    
                    if size[subelem.tag] is not None:
                        raise Exception('xml structure broken at size tag.')
                    size[subelem.tag] = int(subelem.text)

                # option is <xmin>, <ymin>, <xmax>, <ymax>, when subelem is <bndbox>
                for option in subelem:
                    if current_sub == 'bndbox':
                        if bndbox[option.tag] is not None:
                            raise Exception('xml structure corrupted at bndbox tag.')
                        bndbox[option.tag] = int(float(option.text))

                # only after parse the <object> tag
                if bndbox['xmin'] is not None:
                    if object_name is None:
                        raise Exception('xml structure broken at bndbox tag')
                    if current_image_id is None:
                        raise Exception('xml structure broken at bndbox tag')
                    if current_category_id is None:
                        raise Exception('xml structure broken at bndbox tag')
                    bbox = []
                    # x
                    bbox.append(bndbox['xmin'])
                    # y
                    bbox.append(bndbox['ymin'])
                    # w
                    bbox.append(bndbox['xmax'] - bndbox['xmin'])
                    # h
                    bbox.append(bndbox['ymax'] - bndbox['ymin'])
                    print('add annotation with {},{},{},{}'.format(object_name, current_image_id, current_category_id,
                                                                   bbox))
                    addAnnoItem(object_name, current_image_id, current_category_id, bbox)

    print(coco['categories'])
    print(category_set)



def voc2coco(root_dir, txt_file, save_name=None, save_dir=None, save_txt=False):
    '''
    @root_dir: vocdevit dir
    @save_dir: output json file save dir
    @set_type: trainval/train/val/test
    @save_name: save_name.json
    '''
    assert os.path.exists(root_dir)
    xml_dir = os.path.join(root_dir, 'Annotations')
    if not save_dir:
        save_dir = root_dir
    if not save_name:
        sname = 'instances.json'
    else:
        sname = '{}.json'.format(save_name)
    save_path = os.path.join(save_dir, sname)

    txt_list = readTxt2Lines(txt_file)
    parseXmlFiles(xml_dir, txt_list)
    json.dump(coco, open(save_path, 'w'))
    if save_txt:
        writeLines2Txt(voc_txt_lines, os.path.join(save_dir, sname.strip().replace('.json', '.txt')))



if __name__ == '__main__':

    root_dir = r'D:\Dataset\text\CTW'
    txt_file = r'D:\Dataset\text\CTW\ImageSets\Main\val.txt'
    save_name = 'val'
    voc2coco(root_dir, txt_file, save_name)



# coding: utf-8
from __future__ import absolute_import
from __future__ import print_function

from .tools import *
from .decorator import *
from .augment import *
from .dettools import *
from .segtools import vis_segmentation, labelme_json_to_png